from AnnotatedSentence.AnnotatedCorpus import AnnotatedCorpus
from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord
corpus = AnnotatedCorpus("../../Etstur/Deneme")
for sentence in corpus.sentences:
    modified = False
    if isinstance(sentence, AnnotatedSentence):
        groups = sentence.getShallowParseGroups()
        for group in groups:
            word = group.getWord(0)
            if isinstance(word, AnnotatedWord) and word.getUniversalDependency() is None:
                if word.getShallowParse() == "YÜKLEM" and group.wordCount() == 1:
                    word.setUniversalDependency(0, "ROOT")
                    modified = True
    if modified:
        sentence.save()





from abc import abstractmethod

from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence


class SentenceAutoArgument:

    """
    The method should set all the semantic role labels in the sentence. The method assumes that the predicates
    of the sentences were determined previously.

    PARAMETERS
    ----------
    sentence : AnnotatedSentence
        The sentence for which semantic roles will be determined automatically.
    """
    @abstractmethod
    def autoArgument(self, sentence: AnnotatedSentence) -> bool:
        pass



from io import TextIOWrapper

from Corpus.Sentence import Sentence
from MorphologicalAnalysis.FsmMorphologicalAnalyzer import FsmMorphologicalAnalyzer
from PropBank.FramesetList import FramesetList
from WordNet.WordNet import WordNet

from AnnotatedSentence.AnnotatedPhrase import AnnotatedPhrase
from AnnotatedSentence.AnnotatedWord import AnnotatedWord


class AnnotatedSentence(Sentence):

    __fileName: str

    """
    Converts a simple sentence to an annotated sentence

    PARAMETERS
    ----------
    fileOrStr : str
        Simple sentence
    """

    def __init__(self, fileOrStr=None, fileName=None):
        self.words = []
        wordArray = []
        if fileOrStr is not None:
            if fileName is not None:
                self.__fileName = fileName
            if isinstance(fileOrStr, TextIOWrapper):
                line = fileOrStr.readline()
                wordArray = line.rstrip().split(" ")
            elif isinstance(self, str):
                wordArray = fileOrStr.split(" ")
            for word in wordArray:
                if len(word) > 0:
                    self.words.append(AnnotatedWord(word))

    """
    The method constructs all possible shallow parse groups of a sentence.
    
    RETURNS
    -------
    list
        Shallow parse groups of a sentence.
    """
    def getShallowParseGroups(self) -> list:
        shallowParseGroups = []
        previousWord = None
        current = None
        for i in range(self.wordCount()):
            word = self.getWord(i)
            if isinstance(word, AnnotatedWord):
                if previousWord is None:
                    current = AnnotatedPhrase(i, word.getShallowParse())
                else:
                    if isinstance(previousWord, AnnotatedWord) and previousWord.getShallowParse() is not None \
                            and previousWord.getShallowParse() != word.getShallowParse():
                        shallowParseGroups.append(current)
                        current = AnnotatedPhrase(i, word.getShallowParse())
                current.addWord(word)
                previousWord = word
        shallowParseGroups.append(current)
        return shallowParseGroups

    """
    The method checks all words in the sentence and returns true if at least one of the words is annotated with
    PREDICATE tag.

    RETURNS
    -------
    bool
        True if at least one of the words is annotated with PREDICATE tag; False otherwise.
    """

    def containsPredicate(self) -> bool:
        for word in self.words:
            if isinstance(word, AnnotatedWord):
                if word.getArgument() is not None and word.getArgument().getArgumentType() == "PREDICATE":
                    return True
        return False

    def updateConnectedPredicate(self, previousId: str, currentId: str) -> bool:
        modified = False
        for word in self.words:
            if isinstance(word, AnnotatedWord):
                if word.getArgument() is not None and word.getArgument().getId() is not None and \
                        word.getArgument().getId() == previousId:
                    word.setArgument(word.getArgument().getArgumentType() + "$" + currentId)
                    modified = True
        return modified

    """
    The method returns all possible words, which is
    1. Verb
    2. Its semantic tag is assigned
    3. A frameset exists for that semantic tag

    PARAMETERS
    ----------
    framesetList : FramesetList
        Frameset list that contains all frames for Turkish
        
    RETURNS
    -------
    A list of words, which are verbs, semantic tags assigned, and framesetlist assigned for that tag.
    """

    def predicateCandidates(self, framesetList: FramesetList) -> list:
        candidateList = []
        for word in self.words:
            if isinstance(word, AnnotatedWord):
                if word.getParse() is not None and word.getParse().isVerb() and word.getSemantic() is not None \
                        and framesetList.frameExists(word.getSemantic()):
                    candidateList.append(word)
        for i in range(2):
            for j in range(len(self.words) - i - 1):
                annotatedWord = self.words[j]
                nextAnnotatedWord = self.words[j + 1]
                if isinstance(annotatedWord, AnnotatedWord) and isinstance(nextAnnotatedWord, AnnotatedWord):
                    if annotatedWord not in candidateList and nextAnnotatedWord in candidateList \
                            and annotatedWord.getSemantic() is not None \
                            and annotatedWord.getSemantic() == nextAnnotatedWord.getSemantic():
                        candidateList.append(annotatedWord)
        return candidateList

    """
    Returns the nearest predicate to the index'th word in the sentence.

    PARAMETERS
    ----------
    index : int
        Word index
        
    RETURNS
    -------
    str
        The nearest predicate to the index'th word in the sentence.
    """

    def getPredicate(self, index: int) -> str:
        count1 = 0
        count2 = 0
        data = ""
        word = []
        parse = []
        if index < self.wordCount():
            for i in range(self.wordCount()):
                word.append(self.getWord(i))
                parse.append(self.getWord(i).getParse())
            for i in range(index, -1, -1):
                if parse[i] is not None and parse[i].getRootPos() is not None and parse[i].getPos() is not None \
                        and parse[i].getRootPos() == "VERB" and parse[i].getPos() == "VERB":
                    count1 = index - i
                    break
            for i in range(index, self.wordCount() - index):
                if parse[i] is not None and parse[i].getRootPos() is not None and parse[i].getPos() is not None \
                        and parse[i].getRootPos() == "VERB" and parse[i].getPos() == "VERB":
                    count2 = i - index
                    break
            if count1 > count2:
                data = word[count1].getName()
            else:
                data = word[count2].getName()
        return data

    """
    Returns file name of the sentence

    RETURNS
    -------
    str
        File name of the sentence
    """
    def getFileName(self) -> str:
        return self.__fileName

    """
    Removes the i'th word from the sentence

    PARAMETERS
    ----------
    index : int
        Word index
    """

    def removeWord(self, index: int):
        self.words.pop(index)

    """
    Saves the current sentence.
    """
    def save(self):
        self.writeToFile(self.__fileName)

    """
    Creates a list of literal candidates for the i'th word in the sentence. It combines the results of
    1. All possible root forms of the i'th word in the sentence
    2. All possible 2-word expressions containing the i'th word in the sentence
    3. All possible 3-word expressions containing the i'th word in the sentence

    PARAMETERS
    ----------
    wordNet : WordNet
        Turkish wordnet
    fsm : FsmMorphologicalAnalyzer
        Turkish morphological analyzer
    wordIndex : int
        Word index
        
    RETURNS
    -------
    list
        List of literal candidates containing all possible root forms and multiword expressions.
    """

    def constructLiterals(self, wordNet: WordNet, fsm: FsmMorphologicalAnalyzer, wordIndex: int) -> list:
        word = self.getWord(wordIndex)
        possibleLiterals = []
        if isinstance(word, AnnotatedWord):
            morphologicalParse = word.getParse()
            metamorphicParse = word.getMetamorphicParse()
            possibleLiterals.extend(wordNet.constructLiterals(morphologicalParse.getWord().getName(),
                                                              morphologicalParse, metamorphicParse, fsm))
            firstSucceedingWord = None
            secondSucceedingWord = None
            if self.wordCount() > wordIndex + 1:
                firstSucceedingWord = self.getWord(wordIndex + 1)
                if self.wordCount() > wordIndex + 2:
                    secondSucceedingWord = self.getWord(wordIndex + 2)
            if firstSucceedingWord is not None and isinstance(firstSucceedingWord, AnnotatedWord):
                if secondSucceedingWord is not None and isinstance(secondSucceedingWord, AnnotatedWord):
                    possibleLiterals.extend(wordNet.constructIdiomLiterals(fsm, word.getParse(),
                                                                           word.getMetamorphicParse(),
                                                                           firstSucceedingWord.getParse(),
                                                                           firstSucceedingWord.getMetamorphicParse(),
                                                                           secondSucceedingWord.getParse(),
                                                                           secondSucceedingWord.getMetamorphicParse()))
                possibleLiterals.extend(wordNet.constructIdiomLiterals(fsm, word.getParse(), word.getMetamorphicParse(),
                                                                       firstSucceedingWord.getParse(),
                                                                       firstSucceedingWord.getMetamorphicParse()))
        return possibleLiterals

    """
    Creates a list of synset candidates for the i'th word in the sentence. It combines the results of
    1. All possible synsets containing the i'th word in the sentence
    2. All possible synsets containing 2-word expressions, which contains the i'th word in the sentence
    3. All possible synsets containing 3-word expressions, which contains the i'th word in the sentence

    PARAMETERS
    ----------
    wordNet : WordNet
        Turkish wordnet
    fsm : FsmMorphologicalAnalyzer
        Turkish morphological analyzer
    wordIndex : int
        Word index
        
    RETURNS
    -------
    list
        List of synset candidates containing all possible root forms and multiword expressions.
    """
    def constructSynSets(self, wordNet: WordNet, fsm: FsmMorphologicalAnalyzer, wordIndex: int) -> list:
        word = self.getWord(wordIndex)
        possibleSynSets = []
        if isinstance(word, AnnotatedWord):
            morphologicalParse = word.getParse()
            metamorphicParse = word.getMetamorphicParse()
            possibleSynSets.extend(wordNet.constructSynSets(morphologicalParse.getWord().getName(),
                                                            morphologicalParse, metamorphicParse, fsm))
            firstPrecedingWord = None
            secondPrecedingWord = None
            firstSucceedingWord = None
            secondSucceedingWord = None
            if wordIndex > 0:
                firstPrecedingWord = self.getWord(wordIndex - 1)
                if wordIndex > 1:
                    secondPrecedingWord = self.getWord(wordIndex - 2)
            if self.wordCount() > wordIndex + 1:
                firstSucceedingWord = self.getWord(wordIndex + 1)
                if self.wordCount() > wordIndex + 2:
                    secondSucceedingWord = self.getWord(wordIndex + 2)
            if firstPrecedingWord is not None and isinstance(firstPrecedingWord, AnnotatedWord):
                if secondPrecedingWord is not None and isinstance(secondPrecedingWord, AnnotatedWord):
                    possibleSynSets.extend(wordNet.constructIdiomSynSets(fsm, secondPrecedingWord.getParse(),
                                                                         secondPrecedingWord.getMetamorphicParse(),
                                                                         firstPrecedingWord.getParse(),
                                                                         firstPrecedingWord.getMetamorphicParse(),
                                                                         word.getParse(), word.getMetamorphicParse()))
                possibleSynSets.extend(wordNet.constructIdiomSynSets(fsm, firstPrecedingWord.getParse(),
                                                                     firstPrecedingWord.getMetamorphicParse(),
                                                                     word.getParse(), word.getMetamorphicParse()))
            if firstSucceedingWord is not None and isinstance(firstSucceedingWord, AnnotatedWord):
                if secondSucceedingWord is not None and isinstance(secondSucceedingWord, AnnotatedWord):
                    possibleSynSets.extend(wordNet.constructIdiomSynSets(fsm, word.getParse(),
                                                                         word.getMetamorphicParse(),
                                                                         firstSucceedingWord.getParse(),
                                                                         firstSucceedingWord.getMetamorphicParse(),
                                                                         secondSucceedingWord.getParse(),
                                                                         secondSucceedingWord.getMetamorphicParse()))
                possibleSynSets.extend(wordNet.constructIdiomSynSets(fsm, word.getParse(), word.getMetamorphicParse(),
                                                                     firstSucceedingWord.getParse(),
                                                                     firstSucceedingWord.getMetamorphicParse()))
        return possibleSynSets



from AnnotatedSentence.AnnotatedCorpus import AnnotatedCorpus
from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from MorphologicalAnalysis.MorphologicalTag import MorphologicalTag
corpus = AnnotatedCorpus("../../Etstur/Turkish-Phrase")
for sentence in corpus.sentences:
    if isinstance(sentence, AnnotatedSentence):
        modified = False
        ozneGroup = None
        yuklemGroup = None
        groups = sentence.getShallowParseGroups()
        for group in groups:
            word = group.getWord(0)
            if isinstance(word, AnnotatedWord) and word.getShallowParse() == "ÖZNE" and group.wordCount() == 1:
                ozneGroup = group
            if isinstance(word, AnnotatedWord) and word.getShallowParse() == "YÜKLEM" and group.wordCount() == 1:
                yuklemGroup = group
        if ozneGroup is not None and yuklemGroup is not None:
            yuklemWord = yuklemGroup.getWord(0)
            if isinstance(yuklemWord, AnnotatedWord) and yuklemWord.getParse() is not None and not yuklemWord.getParse().containsTag(MorphologicalTag.PASSIVE):
                ozneWord = ozneGroup.getWord(0)
                index = sentence.getIndex(yuklemGroup.getWord(0))
                print(sentence.toString() + "--->" + yuklemGroup.getWord(0).getName() + "--->" + ozneGroup.getWord(0).getName())
                ozneWord.setUniversalDependency(index + 1, "NSUBJ")



from abc import abstractmethod

from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence


class SentenceAutoPredicate:

    """
    The method should set determine all predicates in the sentence.

    PARAMETERS
    ----------
    sentence : AnnotatedSentence
        The sentence for which predicates will be determined automatically.
    """
    @abstractmethod
    def autoPredicate(self, sentence: AnnotatedSentence) -> bool:
        pass



from MorphologicalAnalysis.FsmMorphologicalAnalyzer import FsmMorphologicalAnalyzer
from WordNet.WordNet import WordNet

from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from AnnotatedSentence.AutoProcessor.AutoSemantic.SentenceAutoSemantic import SentenceAutoSemantic


class TurkishAutoSemantic(SentenceAutoSemantic):

    __turkishWordNet: WordNet
    __fsm: FsmMorphologicalAnalyzer

    """
    Constructor for the {@link TurkishSentenceAutoSemantic} class. Gets the Turkish wordnet and Turkish fst based
    morphological analyzer from the user and sets the corresponding attributes.

    PARAMETERS
    ----------
    turkishWordNet : WordNet
        Turkish wordnet
    fsm : FsmMorphologicalAnalyzer
        Turkish morphological analyzer
    """
    def __init__(self, turkishWordNet: WordNet, fsm: FsmMorphologicalAnalyzer):
        self.__turkishWordNet = turkishWordNet
        self.__fsm = fsm

    """
    The method checks
    1. the previous two words and the current word; the previous, current and next word, current and the next
    two words for a three word multiword expression that occurs in the Turkish wordnet.
    2. the previous word and current word; current word and the next word for a two word multiword expression that
    occurs in the Turkish wordnet.
    3. the current word
    if it has only one sense. If there is only one sense for that multiword expression or word; it sets that sense.

    PARAMETERS
    ----------
    sentence : AnnotatedSentence 
        The sentence for which word sense disambiguation will be determined automatically.
    """
    def autoLabelSingleSemantics(self, sentence: AnnotatedSentence):
        for i in range(sentence.wordCount()):
            current = sentence.getWord(i)
            if i > 1:
                twoPrevious = sentence.getWord(i - 2)
            if i > 0:
                previous = sentence.getWord(i - 1)
            if i != sentence.wordCount() - 1:
                next = sentence.getWord(i + 1)
            if i < sentence.wordCount() - 2:
                twoNext = sentence.getWord(i + 2)
            if isinstance(current, AnnotatedWord) and current.getSemantic() is None and current.getParse() is not None:
                if twoPrevious is not None and isinstance(twoPrevious, AnnotatedWord) \
                        and twoPrevious.getParse() is not None and isinstance(previous, AnnotatedWord) \
                        and previous.getParse() is not None:
                    idioms = self.__turkishWordNet.constructIdiomSynSets(self.__fsm, twoPrevious.getParse(),
                                                                         twoPrevious.getMetamorphicParse(),
                                                                         previous.getParse(),
                                                                         previous.getMetamorphicParse(),
                                                                         current.getParse(),
                                                                         current.getMetamorphicParse())
                    if len(idioms) == 1:
                        current.setSemantic(idioms[0].getId())
                        continue
                if previous is not None and isinstance(previous, AnnotatedWord) \
                        and previous.getParse() is not None and next is not None and isinstance(next, AnnotatedWord) \
                        and next.getParse() is not None:
                    idioms = self.__turkishWordNet.constructIdiomSynSets(self.__fsm, previous.getParse(),
                                                                         previous.getMetamorphicParse(),
                                                                         current.getParse(),
                                                                         current.getMetamorphicParse(),
                                                                         next.getParse(),
                                                                         next.getMetamorphicParse())
                    if len(idioms) == 1:
                        current.setSemantic(idioms[0].getId())
                        continue
                if next is not None and isinstance(next, AnnotatedWord) \
                        and next.getParse() is not None and twoNext is not None and isinstance(twoNext, AnnotatedWord) \
                        and twoNext.getParse() is not None:
                    idioms = self.__turkishWordNet.constructIdiomSynSets(self.__fsm, current.getParse(),
                                                                         current.getMetamorphicParse(),
                                                                         next.getParse(),
                                                                         next.getMetamorphicParse(),
                                                                         twoNext.getParse(),
                                                                         twoNext.getMetamorphicParse())
                    if len(idioms) == 1:
                        current.setSemantic(idioms[0].getId())
                        continue
                if previous is not None and isinstance(previous, AnnotatedWord) and previous.getParse() is not None:
                    idioms = self.__turkishWordNet.constructIdiomSynSets(self.__fsm, previous.getParse(),
                                                                         previous.getMetamorphicParse(),
                                                                         current.getParse(),
                                                                         current.getMetamorphicParse())
                    if len(idioms) == 1:
                        current.setSemantic(idioms[0].getId())
                        continue
                if next is not None and isinstance(next, AnnotatedWord) and next.getParse() is not None:
                    idioms = self.__turkishWordNet.constructIdiomSynSets(self.__fsm, current.getParse(),
                                                                         current.getMetamorphicParse(),
                                                                         next.getParse(),
                                                                         next.getMetamorphicParse())
                    if len(idioms) == 1:
                        current.setSemantic(idioms[0].getId())
                        continue
                meanings = self.__turkishWordNet.constructSynSets(current.getParse().getWord().getName(),
                                                                  current.getParse(), current.getMetamorphicParse(),
                                                                  self.__fsm)
                if current.getSemantic() is None and len(meanings) == 1:
                    current.setSemantic(meanings[0].getId())



from AnnotatedSentence.AnnotatedCorpus import AnnotatedCorpus
from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord
corpus = AnnotatedCorpus("../../Etstur/Deneme")
for sentence in corpus.sentences:
    if isinstance(sentence, AnnotatedSentence):
        modified = False
        groupFound = None
        groups = sentence.getShallowParseGroups()
        for group in groups:
            word = group.getWord(0)
            if isinstance(word, AnnotatedWord) and word.getShallowParse() == "YÜKLEM" and group.wordCount() == 1:
                groupFound = group
        lastWord = sentence.getWord(sentence.wordCount() - 1)
        if lastWord.getName() == "." and groupFound is not None:
            index = sentence.getIndex(groupFound.getWord(0))
            print(sentence.toString() + "--->" + groupFound.getWord(0).getName() + "--->" + index.__str__())
            lastWord.setUniversalDependency(index + 1, "PUNCT")
            modified = True
    if modified:
        sentence.save()



from Corpus.Sentence import Sentence


class AnnotatedPhrase(Sentence):

    __wordIndex: int
    __tag: str

    """
    Constructor for AnnotatedPhrase. AnnotatedPhrase stores information about phrases such as
    Shallow Parse phrases or named entity phrases.
    
    PARAMETERS
    ----------
    wordIndex : int
        Starting index of the first word in the phrase w.r.t. original sentence the phrase occurs.
    tag : str
        Tag of the phrase. Corresponds to the shallow parse or named entity tag.
    """
    def __init__(self, wordIndex: int, tag: str):
        self.__wordIndex = wordIndex
        self.__tag = tag

    """
    Accessor for the wordIndex attribute.
    
    RETURNS
    -------
    int
        Starting index of the first word in the phrase w.r.t. original sentence the phrase occurs.
    """
    def getWordIndex(self) -> int:
        return self.__wordIndex

    """
    Accessor for the tag attribute.
    
    RETURNS
    -------
    str
        Tag of the phrase. Corresponds to the shallow parse or named entity tag.
    """
    def getTag(self) -> str:
        return self.__tag



from Corpus.WordFormat import WordFormat
from DependencyParser.UniversalDependencyRelation import UniversalDependencyRelation
from Dictionary.Word import Word
from MorphologicalAnalysis.FsmParse import FsmParse
from MorphologicalAnalysis.MetamorphicParse import MetamorphicParse
from MorphologicalAnalysis.MorphologicalParse import MorphologicalParse
from MorphologicalAnalysis.MorphologicalTag import MorphologicalTag
from NamedEntityRecognition.Gazetteer import Gazetteer
from NamedEntityRecognition.NamedEntityType import NamedEntityType
from PropBank.Argument import Argument

import re

from AnnotatedSentence.ViewLayerType import ViewLayerType


class AnnotatedWord(Word):
    __parse: MorphologicalParse
    __metamorphicParse: MetamorphicParse
    __semantic: str
    __namedEntityType: NamedEntityType
    __argument: Argument
    __shallowParse: str
    __universalDependency: UniversalDependencyRelation

    """
    Constructor for the AnnotatedWord class. Gets the word with its annotation layers as input and sets the
    corresponding layers.

    PARAMETERS
    ----------
    word : str
        Input word with annotation layers
    """

    def __init__(self, word: str, layerType=None):
        self.__parse = None
        self.__metamorphicParse = None
        self.__semantic = None
        self.__namedEntityType = None
        self.__argument = None
        self.__shallowParse = None
        self.__universalDependency = None
        if layerType is None:
            splitLayers = re.compile("[{}]").split(word)
            for layer in splitLayers:
                if len(layer) == 0:
                    continue
                if "=" not in layer:
                    self.name = layer
                    continue
                layerType = layer[:layer.index("=")]
                layerValue = layer[layer.index("=") + 1:]
                if layerType == "turkish":
                    self.name = layerValue
                elif layerType == "morphologicalAnalysis":
                    self.__parse = MorphologicalParse(layerValue)
                elif layerType == "metaMorphemes":
                    self.__metamorphicParse = MetamorphicParse(layerValue)
                elif layerType == "namedEntity":
                    self.__namedEntityType = NamedEntityType.getNamedEntityType(layerValue)
                elif layerType == "propbank":
                    self.__argument = Argument(layerValue)
                elif layerType == "shallowParse":
                    self.__shallowParse = layerValue
                elif layerType == "semantics":
                    self.__semantic = layerValue
                elif layerType == "universalDependency":
                    values = layerValue.split("$")
                    self.__universalDependency = UniversalDependencyRelation(int(values[0]), values[1])
        elif isinstance(layerType, NamedEntityType):
            super().__init__(word)
            self.__namedEntityType = layerType
            self.__argument = Argument("NONE")
        elif isinstance(layerType, MorphologicalParse):
            super().__init__(word)
            self.__parse = layerType
            self.__namedEntityType = NamedEntityType.NONE
            self.__argument = Argument("NONE")
        elif isinstance(layerType, FsmParse):
            super().__init__(word)
            self.__parse = layerType
            self.__namedEntityType = NamedEntityType.NONE
            self.setMetamorphicParse(layerType.withList())
            self.__argument = Argument("NONE")

    """
    Converts an AnnotatedWord to string. For each annotation layer, the method puts a left brace, layer name,
    equal sign and layer value finishing with right brace.

    RETURNS
    -------
    str
        String form of the AnnotatedWord.
    """

    def __str__(self) -> str:
        result = "{turkish=" + self.name + "}"
        if self.__parse is not None:
            result = result + "{morphologicalAnalysis=" + self.__parse.__str__() + "}"
        if self.__metamorphicParse is not None:
            result = result + "{metaMorphemes=" + self.__metamorphicParse.__str__() + "}"
        if self.__semantic is not None:
            result = result + "{semantics=" + self.__semantic + "}"
        if self.__namedEntityType is not None:
            result = result + "{namedEntity=" + self.__namedEntityType.__str__() + "}"
        if self.__argument is not None:
            result = result + "{propbank=" + self.__argument.__str__() + "}"
        if self.__shallowParse is not None:
            result = result + "{shallowParse=" + self.__shallowParse + "}"
        if self.__universalDependency is not None:
            result = result + "{universalDependency=" + self.__universalDependency.to().__str__() + "$" + self.__universalDependency.__str__() + "}"
        return result

    """
    Returns the value of a given layer.

    PARAMETERS
    ----------
    viewLayerType : ViewLayerType
        Layer for which the value questioned.
        
    RETURNS
    -------
    str
        The value of the given layer.
    """

    def getLayerInfo(self, viewLayerType: ViewLayerType) -> str:
        if viewLayerType == ViewLayerType.INFLECTIONAL_GROUP:
            if self.__parse is not None:
                return self.__parse.__str__()
        elif viewLayerType == ViewLayerType.META_MORPHEME:
            if self.__metamorphicParse is not None:
                return self.__metamorphicParse.__str__()
        elif viewLayerType == ViewLayerType.SEMANTICS:
            return self.__semantic
        elif viewLayerType == ViewLayerType.NER:
            if self.__namedEntityType is not None:
                return self.__namedEntityType.__str__()
        elif viewLayerType == ViewLayerType.SHALLOW_PARSE:
            return self.__shallowParse
        elif viewLayerType == ViewLayerType.TURKISH_WORD:
            return self.name
        elif viewLayerType == ViewLayerType.PROPBANK:
            if self.__argument is not None:
                return self.__argument.__str__()
        elif viewLayerType == ViewLayerType.DEPENDENCY:
            if self.__universalDependency is not None:
                return self.__universalDependency.to().__str__() + "$" + self.__universalDependency.__str__()
        else:
            return None

    """
    Returns the morphological parse layer of the word.
    
    RETURNS
    -------
    MorphologicalParse
        The morphological parse of the word.
    """

    def getParse(self) -> MorphologicalParse:
        return self.__parse

    """
    Sets the morphological parse layer of the word.
    
    PARAMETERS
    ----------
    parseString : str
        The new morphological parse of the word in string form.
    """

    def setParse(self, parseString: MorphologicalParse):
        if parseString is not None:
            self.__parse = MorphologicalParse(parseString)
        else:
            self.__parse = None

    """
    Returns the metamorphic parse layer of the word.
    
    RETURNS
    -------
    MetamorphicParse
        The metamorphic parse of the word.
    """

    def getMetamorphicParse(self) -> MetamorphicParse:
        return self.__metamorphicParse

    """
    Sets the metamorphic parse layer of the word.

    PARAMETERS
    ----------
    parseString : str
        The new metamorphic parse of the word in string form.
    """

    def setMetamorphicParse(self, parseString: str):
        self.__metamorphicParse = MetamorphicParse(parseString)

    """
    Returns the semantic layer of the word.

    RETURNS
    -------
    str
        Sense id of the word.
    """

    def getSemantic(self) -> str:
        return self.__semantic

    """
    Sets the semantic layer of the word.

    PARAMETERS
    ----------
    semantic : str
        New sense id of the word.
    """

    def setSemantic(self, semantic: str):
        self.__semantic = semantic

    """
    Returns the named entity layer of the word.

    RETURNS
    -------
    NamedEntityType
        Named entity tag of the word.
    """

    def getNamedEntityType(self) -> NamedEntityType:
        return self.__namedEntityType

    """
    Sets the named entity layer of the word.

    PARAMETERS
    ----------
    namedEntity : str
        New named entity tag of the word.
    """

    def setNamedEntityType(self, namedEntity: str):
        if namedEntity is not None:
            self.__namedEntityType = NamedEntityType.getNamedEntityType(namedEntity)
        else:
            self.__namedEntityType = None

    """
    Returns the semantic role layer of the word.
    
    RETURNS
    -------
    Argument
        Semantic role tag of the word.
    """

    def getArgument(self) -> Argument:
        return self.__argument

    """
    Sets the semantic role layer of the word.

    PARAMETERS
    ----------
    argument : Argument
        New semantic role tag of the word.
    """

    def setArgument(self, argument: str):
        if self.__argument is not None:
            self.__argument = Argument(argument)
        else:
            self.__argument = None

    """
    Returns the shallow parse layer of the word.

    RETURNS
    -------
    str
        Shallow parse tag of the word.
    """

    def getShallowParse(self) -> str:
        return self.__shallowParse

    """
    Sets the shallow parse layer of the word.

    PARAMETERS
    ----------
    parse : str
        New shallow parse tag of the word.
    """

    def setShallowParse(self, parse: str):
        self.__shallowParse = parse

    """
    Returns the universal dependency layer of the word.
    
    RETURNS
    -------
    UniversalDependencyRelation
        Universal dependency relation of the word.
    """
    def getUniversalDependency(self) -> UniversalDependencyRelation:
        return self.__universalDependency

    """
    Sets the universal dependency layer of the word.

    PARAMETERS
    ----------
    to : int
        to Word related to.
    dependencyType : str
        type of dependency the word is related to.
    """
    def setUniversalDependency(self, to: int, dependencyType: str):
        self.__universalDependency = UniversalDependencyRelation(to, dependencyType)

    def getFormattedString(self, wordFormat: WordFormat):
        if wordFormat == WordFormat.SURFACE:
            return self.name
        return self.name

    def checkGazetteer(self, gazetteer: Gazetteer):
        wordLowercase = self.name.lower()
        if gazetteer.contains(wordLowercase) and self.__parse.containsTag(MorphologicalTag.PROPERNOUN):
            self.setNamedEntityType(gazetteer.getName())
        if "'" in wordLowercase and gazetteer.contains(wordLowercase[:wordLowercase.index("'")]) and \
                self.__parse.containsTag(MorphologicalTag.PROPERNOUN):
            self.setNamedEntityType(gazetteer.getName())



from MorphologicalAnalysis.MorphologicalTag import MorphologicalTag

from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from AnnotatedSentence.AutoProcessor.AutoArgument.SentenceAutoArgument import SentenceAutoArgument


class TurkishSentenceAutoArgument(SentenceAutoArgument):

    """
    Given the sentence for which the predicate(s) were determined before, this method automatically assigns
    semantic role labels to some/all words in the sentence. The method first finds the first predicate, then assuming
    that the shallow parse tags were preassigned, assigns ÖZNE tagged words ARG0; NESNE tagged words ARG1. If the
    verb is in passive form, ÖZNE tagged words are assigned as ARG1.

    PARAMETERS
    ----------
    sentence : AnnotatedSentence
        The sentence for which semantic roles will be determined automatically.

    RETURNS
    -------
    bool
        If the method assigned at least one word a semantic role label, the method returns true; false otherwise.
    """
    def autoArgument(self, sentence: AnnotatedSentence) -> bool:
        modified = False
        predicateId = None
        for i in range(sentence.wordCount()):
            word = sentence.getWord(i)
            if isinstance(word, AnnotatedWord):
                if word.getArgument() is not None and word.getArgument().getArgumentType() == "PREDICATE":
                    predicateId = word.getArgument().getId()
                    break
        if predicateId is not None:
            for i in range(sentence.wordCount()):
                word = sentence.getWord(i)
                if isinstance(word, AnnotatedWord) and word.getArgument() is None:
                    if word.getShallowParse() is not None and word.getShallowParse() == "ÖZNE":
                        if word.getParse() is not None and word.getParse().containsTag(MorphologicalTag.PASSIVE):
                            word.setArgument("ARG1$" + predicateId)
                        else:
                            word.setArgument("ARG0$" + predicateId)
                        modified = True
                    else:
                        if word.getShallowParse() is not None and word.getShallowParse() == "NESNE":
                            word.setArgument("ARG1$" + predicateId)
                            modified = True
        return modified



from PropBank.FramesetList import FramesetList

from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from AnnotatedSentence.AutoProcessor.AutoPredicate.SentenceAutoPredicate import SentenceAutoPredicate


class TurkishSentenceAutoPredicate(SentenceAutoPredicate):

    __framesetList: FramesetList

    """
    Constructor for TurkishSentenceAutoPredicate. Gets the FrameSets as input from the user, and sets
    the corresponding attribute.

    PARAMETERS
    ----------
    framesetList : FramesetList
        FramesetList containing the Turkish propbank frames.
    """
    def __init__(self, framesetList: FramesetList):
        self.__framesetList = framesetList

    """
    The method uses predicateCandidates method to predict possible predicates. For each candidate, it sets for that
    word PREDICATE tag.

    PARAMETERS
    ----------
    sentence : AnnotatedSentence
        The sentence for which predicates will be determined automatically.

    RETURNS
    -------
    bool
        If at least one word has been tagged, true; false otherwise.
    """
    def autoPredicate(self, sentence: AnnotatedSentence) -> bool:
        candidateList = sentence.predicateCandidates(self.__framesetList)
        for word in candidateList:
            if isinstance(word, AnnotatedWord):
                word.setArgument("PREDICATE$" + word.getSemantic())
        if len(candidateList) > 0:
            return True
        return False



from Corpus.Corpus import Corpus
import os

from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord


class AnnotatedCorpus(Corpus):

    """
    A constructor of AnnotatedCorpus class which reads all AnnotatedSentence files with the file
    name satisfying the given pattern inside the given folder. For each file inside that folder, the constructor
    creates an AnnotatedSentence and puts in inside the list parseTrees.

    PARAMETERS
    ----------
    folder : str
        Folder where all sentences reside.
    pattern : str
        File pattern such as "." ".train" ".test".
    """
    def __init__(self, folder: str, pattern: str = None):
        self.sentences = []
        for root, dirs, files in os.walk(folder):
            for file in files:
                fileName = os.path.join(root, file)
                if pattern is None or pattern in fileName:
                    f = open(fileName, "r", encoding='utf8')
                    sentence = AnnotatedSentence(f, fileName)
                    self.sentences.append(sentence)

    """
    The method traverses all words in all sentences and prints the words which do not have a morphological analysis.
    """
    def checkMorphologicalAnalysis(self):
        for i in range(self.sentenceCount()):
            sentence = self.getSentence(i)
            if isinstance(sentence, AnnotatedSentence):
                for j in range(sentence.wordCount()):
                    word = sentence.getWord(j)
                    if isinstance(word, AnnotatedWord):
                        if word.getParse() is None:
                            print("Morphological Analysis does not exist for sentence " + sentence.getFileName())
                            break

    """
    The method traverses all words in all sentences and prints the words which do not have named entity annotation.
    """
    def checkNer(self):
        for i in range(self.sentenceCount()):
            sentence = self.getSentence(i)
            if isinstance(sentence, AnnotatedSentence):
                for j in range(sentence.wordCount()):
                    word = sentence.getWord(j)
                    if isinstance(word, AnnotatedWord):
                        if word.getNamedEntityType() is None:
                            print("NER annotation does not exist for sentence " + sentence.getFileName())
                            break

    """
    The method traverses all words in all sentences and prints the words which do not have shallow parse annotation.
    """
    def checkShallowParse(self):
        for i in range(self.sentenceCount()):
            sentence = self.getSentence(i)
            if isinstance(sentence, AnnotatedSentence):
                for j in range(sentence.wordCount()):
                    word = sentence.getWord(j)
                    if isinstance(word, AnnotatedWord):
                        if word.getShallowParse() is None:
                            print("Shallow parse annotation does not exist for sentence " + sentence.getFileName())
                            break

    """
    The method traverses all words in all sentences and prints the words which do not have sense annotation.
    """
    def checkSemantic(self):
        for i in range(self.sentenceCount()):
            sentence = self.getSentence(i)
            if isinstance(sentence, AnnotatedSentence):
                for j in range(sentence.wordCount()):
                    word = sentence.getWord(j)
                    if isinstance(word, AnnotatedWord):
                        if word.getSemantic() is None:
                            print("Semantic annotation does not exist for sentence " + sentence.getFileName())
                            break
